package com.ibm.sysmgt.raidmgr.cim.provider.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.cim.provider.*;

import com.sun.wbem.cim.*;
import com.sun.wbem.provider.CIMProvider;
import com.sun.wbem.provider20.Authorizable;
import com.sun.wbem.provider20.InstanceProvider;
import com.sun.wbem.client.ProviderCIMOMHandle;
import com.sun.wbem.query.SelectExp;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

public class SNMPProvider extends RAIDInstanceProviderBase
{
   public static final String CIM_SNMP_AGENT_CREATION_CLASS_NAME = "IBMPSG_SNMPRAIDAgent";

   public SNMPProvider()
   {
      super("SNMPProvider:");
      instanceLog.println(logHeader + "SNMPProvider()");
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ") Names Only");

      SetInteractiveWSAD();

      Vector cops = new Vector();
      Vector instances = null;

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(Adapter.CIM_SNMP_CREATION_CLASS_NAME))
         instances = enumSNMPAdapters(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(PhysicalDevice.CIM_SNMP_CREATION_CLASS_NAME))
         instances = enumSNMPPhysicalDevices(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(LogicalDrive.CIM_SNMP_CREATION_CLASS_NAME))
         instances = enumSNMPLogicalDevices(system, cc);
      else if (op.getObjectName().equalsIgnoreCase(CIM_SNMP_AGENT_CREATION_CLASS_NAME))
         instances = enumSNMPAgents(system, cc);
      else
      {
         instanceLog.println(logHeader + "Does not provide names of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }

      for (Enumeration e = instances.elements(); e.hasMoreElements();)
      {
         CIMInstance ci = (CIMInstance)e.nextElement();
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(ci.getKeyValuePairs());
         cops.addElement(cop);
      }

      RestoreInteractiveWSAD();
      return cops;
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");

      SetInteractiveWSAD();

      if (isConnectedToAgent() == false)
      {
         instanceLog.println(logHeader + "Cannot enumerate classes, not connected to agent.");
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(Adapter.CIM_SNMP_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return enumSNMPAdapters(system, cc);
      }
      else if (op.getObjectName().equalsIgnoreCase(PhysicalDevice.CIM_SNMP_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return enumSNMPPhysicalDevices(system, cc);
      }
      else if (op.getObjectName().equalsIgnoreCase(LogicalDrive.CIM_SNMP_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return enumSNMPLogicalDevices(system, cc);
      }
      else if (op.getObjectName().equalsIgnoreCase(CIM_SNMP_AGENT_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return enumSNMPAgents(system, cc);
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public CIMInstance getInstance(CIMObjectPath op, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");

      SetInteractiveWSAD();

      if (isConnectedToAgent() == false)
      {
         instanceLog.println(logHeader + "Can't get class of type "  + op.getObjectName() + ", not connected to agent.");
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      RaidSystem system = null;
      try
      {
         Object[] args = null;
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(Adapter.CIM_SNMP_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return getSNMPAdapter(system, cc, op);
      }
      else if (op.getObjectName().equalsIgnoreCase(PhysicalDevice.CIM_SNMP_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return getSNMPPhysicalDevice(system, cc, op);
      }
      else if (op.getObjectName().equalsIgnoreCase(LogicalDrive.CIM_SNMP_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();
         return getSNMPLogicalDevice(system, cc, op);
      }
      else if (op.getObjectName().equalsIgnoreCase(CIM_SNMP_AGENT_CREATION_CLASS_NAME))
      {
         RestoreInteractiveWSAD();      
         return getSNMPAgent(system, cc, op);
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
   }

   // **********************
   // SNMP Adapters
   // **********************
   public Vector enumSNMPAdapters(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter)e.nextElement();
         instances.addElement(adapter.createCIMSNMPInstance(cc.newInstance()));
      }

      return instances;
   }   

   public CIMInstance getSNMPAdapter(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         UnsignedInt32 deviceID = ((UnsignedInt32)getKeyValue(op, "ControllerID"));
         int adapterID = Adapter.adjustedIDToAdapterID(deviceID.intValue());
         Adapter adapter = (Adapter) system.getAdapter(adapterID);
         return adapter.createCIMSNMPInstance(cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

   // **********************
   // SNMP Physical Devices
   // **********************
   public Vector enumSNMPPhysicalDevices(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {
         Adapter adapter = (Adapter) e.nextElement();
         for (Enumeration e2 = adapter.getPhysicalDeviceCollection(null).elements(); e2.hasMoreElements();)
         {
            PhysicalDevice pd = (PhysicalDevice)e2.nextElement();            
            instances.addElement(pd.createCIMSNMPInstance(cc.newInstance()));
         }         
      }

      return instances;
   }   

   public CIMInstance getSNMPPhysicalDevice(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {         
         int deviceID = ((UnsignedInt32)getKeyValue(op, "DeviceID")).intValue();
         int controllerID = ((UnsignedInt32)getKeyValue(op, "ControllerID")).intValue();
         int channelID = ((UnsignedInt32)getKeyValue(op, "ChannelID")).intValue();

         // Convert from display IDs to device IDs
         controllerID = Adapter.adjustedIDToAdapterID(controllerID);
         channelID = Channel.displayIDToChannelID(Integer.toString(channelID));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         for (Enumeration e = adapter.getPhysicalDeviceCollection(null).elements(); e.hasMoreElements();)
         {
            PhysicalDevice pd = (PhysicalDevice)e.nextElement();
            if (pd.getChannelID() == channelID && pd.getDeviceID() == deviceID)
               return pd.createCIMSNMPInstance(cc.newInstance());
         }
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      
      throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);      
   }

   // **********************
   // Logical Devices
   // **********************
   public Vector enumSNMPLogicalDevices(RaidSystem system, CIMClass cc) throws CIMException
   {
      Vector instances = new Vector();

      for (Enumeration e = system.enumerateAdapters(); e.hasMoreElements();)
      {         
         Adapter adapter = (Adapter)e.nextElement();
         for (Enumeration e2 = adapter.enumerateLogicalDrives(); e2.hasMoreElements();)
         {            
            LogicalDrive ld = (LogicalDrive)e2.nextElement();
            if (ld.getState() != Constants.FRE)
               instances.addElement(ld.createCIMSNMPInstance(cc.newInstance()));
         }
      }

      return instances;
   }   

   public CIMInstance getSNMPLogicalDevice(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      try
      {
         //int ldID = Integer.parseInt((String)getKeyValue(op, "LogicalDriveNumber"));
         int ldID = ((UnsignedInt32)getKeyValue(op, "LogicalDriveNumber")).intValue();
         int controllerID = ((UnsignedInt32)getKeyValue(op, "ControllerID")).intValue();

         // Convert from display IDs to device IDs
         controllerID = Adapter.adjustedIDToAdapterID(controllerID);
         ldID = LogicalDrive.displayIDToLogicalDriveID(Integer.toString(ldID));

         Adapter adapter = (Adapter)system.getAdapter(controllerID);
         return adapter.getLogicalDrive(ldID).createCIMSNMPInstance(cc.newInstance());
      }
      catch (ClassCastException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
      catch (NumberFormatException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }      
      catch (NullPointerException e)
      {
         e.printStackTrace(instanceLog);
         throw new CIMException(CIMException.CIM_ERR_NOT_FOUND);
      }
   }

   // **********************
   // SNMP Agents
   // **********************
   public Vector enumSNMPAgents(RaidSystem system, CIMClass cc) throws CIMException
   {      
      Vector instances = new Vector();     

      instances.addElement(createSNMPAgentInstance(cc.newInstance()));

      return instances;
   }   

   public CIMInstance getSNMPAgent(RaidSystem system, CIMClass cc, CIMObjectPath op) throws CIMException
   {
      return createSNMPAgentInstance(cc.newInstance());
   }

   private CIMInstance createSNMPAgentInstance(CIMInstance instance)
   {
      instance.setProperty("AgentID", new CIMValue(JCRMUtil.getNLSString("infoSystemJCRMAgentVersion")));
      instance.setProperty("AgentCompany", new CIMValue(JCRMUtil.getNLSString("cimManufacturer")));
      instance.setProperty("AgentVersion", new CIMValue(new String(Constants.JCRM_USER_VERSION)));
      instance.setProperty("AgentBuildDate", new CIMValue(new String("00000000")));

      StringTokenizer st = new StringTokenizer(Constants.JCRM_USER_VERSION, ".");

      try
      {      
         instance.setProperty("AgentVersionMajor", new CIMValue(new UnsignedInt16(Integer.parseInt(st.nextToken()))));
      }
      catch (NoSuchElementException e)
      {
         e.printStackTrace(instanceLog);
         instance.setProperty("AgentVersionMajor", new CIMValue(new UnsignedInt16(0)));
      }

      try
      {      
         instance.setProperty("AgentVersionMinor", new CIMValue(new UnsignedInt16(Integer.parseInt(st.nextToken()))));
      }
      catch (NoSuchElementException e)
      {
         e.printStackTrace(instanceLog);
         instance.setProperty("AgentVersionMinor", new CIMValue(new UnsignedInt16(0)));
      }

      return instance;
   }

    public static void main(String [] args)
    {
        SNMPProvider provider = new SNMPProvider();
        Object [] args1 = null;
        try
        {
            RaidSystem system = provider.getRaidSystem();
            System.out.println(system);
            System.out.println("Controller count = " + system.getAdapterCount());
            System.exit(0);
        }
        catch (RemoteException ex)
        {
            System.out.println("Error accessing the ServeRAID Manager agent. Exception: " + ex);
        }
    }
}
