package com.ibm.sysmgt.raidmgr.cim.provider.instance;

import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.cim.provider.*;

import com.sun.wbem.cim.*;
import com.sun.wbem.provider.CIMProvider;
import com.sun.wbem.provider20.Authorizable;
import com.sun.wbem.provider20.InstanceProvider;
import com.sun.wbem.client.ProviderCIMOMHandle;
import com.sun.wbem.query.SelectExp;

import java.util.*;
import java.io.*;
import java.rmi.RemoteException;

public class ServeRaidOverallStatusProvider extends RAIDInstanceProviderBase
{
   public static final String CIM_CREATION_CLASS_NAME = "IBMPSG_ServeRAIDOverallStatus";
   public static final String SETTING_ID = "ServeRAID Subsystem";

   public ServeRaidOverallStatusProvider()
   {
      super("ServeRaidOverallStatusProvider:");
      instanceLog.println(logHeader + "ServeRaidOverallStatusProvider()");
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ") Names Only");

      SetInteractiveWSAD();

      if (isConnectedToAgent() == false)
      {
         instanceLog.println(logHeader + "Cannot enumerate classes, not connected to agent.");
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         Vector cops = new Vector();
         CIMInstance instance = cc.newInstance();         

         instance.setProperty("SettingID", new CIMValue(SETTING_ID));
         
         CIMObjectPath cop = new CIMObjectPath(cc.getName(), op.getNameSpace());
         cop.setKeys(instance.getKeyValuePairs());
         cops.addElement(cop);

         RestoreInteractiveWSAD();
         return cops;
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide names of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public Vector enumInstances(CIMObjectPath op, boolean deep, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "enumInstances(" + op.getObjectName() + ")");

      SetInteractiveWSAD();

      if (isConnectedToAgent() == false)
      {
         instanceLog.println(logHeader + "Cannot enumerate classes, not connected to agent.");
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      RaidSystem system = null;
      try
      {         
         system = (RaidSystem)getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         Vector instances = new Vector();
         CIMInstance instance = cc.newInstance();         
         int overallStatus = system.getOverallStatus();

         instance.setProperty("SettingID", new CIMValue(SETTING_ID));
         instance.setProperty("OverallStatus", new CIMValue(new UnsignedInt16(overallStatus)));

         instances.addElement(instance);

         RestoreInteractiveWSAD();
         return instances;
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }
   }

   public CIMInstance getInstance(CIMObjectPath op, CIMClass cc, boolean localOnly) throws CIMException
   {
      instanceLog.println(logHeader + "getInstance(" + op.getObjectName() + ")");

      SetInteractiveWSAD();

      if (isConnectedToAgent() == false)
      {
         instanceLog.println(logHeader + "Can't get class of type "  + op.getObjectName() + ", not connected to agent.");
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      RaidSystem system = null;
      try
      {
         Object[] args = null;
         system = getRaidSystem();
      }
      catch (RemoteException e)
      {
         instanceLog.println(logHeader + "Error accessing the ServeRAID Manager agent. Exception: " + e);
         RestoreInteractiveWSAD();
         throw new CIMException(CIMException.CIM_ERR_FAILED);
      }

      if (op.getObjectName().equalsIgnoreCase(CIM_CREATION_CLASS_NAME))
      {
         CIMInstance instance = cc.newInstance();
         int overallStatus = system.getOverallStatus();

         instance.setProperty("SettingID", new CIMValue(SETTING_ID));
         instance.setProperty("OverallStatus", new CIMValue(new UnsignedInt16(overallStatus)));

         RestoreInteractiveWSAD();
         return instance;
      }
      else
      {
         instanceLog.println(logHeader + "Does not provide instances of class " + op.getObjectName());
         RestoreInteractiveWSAD();
         throw (new CIMException(CIMException.CIM_ERR_NOT_SUPPORTED));
      }      
   }

    public static void main(String [] args)
    {
        ServeRaidOverallStatusProvider provider = new ServeRaidOverallStatusProvider();
        Object [] args1 = null;
        try
        {
            RaidSystem system = provider.getRaidSystem();
            System.out.println(system);
            System.out.println("Controller count = " + system.getAdapterCount());
            System.exit(0);
        }
        catch (RemoteException ex)
        {
            System.out.println("Error accessing the ServeRAID Manager agent. Exception: " + ex);
        }
    }
}
