package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  FibreInterface.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                      DAK           02/07/02
//

import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;

public class FibreInterface implements Serializable, Constants
{
    static final long serialVersionUID = 7438199997873629401L;

    // Fibre Topology's
    public static final int ARBITRATED_LOOP = 1;
    public static final int FABRIC_LOOP = 2;
    public static final int FABRIC = 3;
    public static final int POINT_TO_POINT =4;
    public static final int UNKNOWN = 5;

    private int     interfaceType;
    private int     topology;
    private int     channel;
    private int     hardAddress;
    private int     loopId;
    private String  portId;
    private String  portName;
    private String  nodeName;
    private static final int loopIdTable[] = { 
                         0xef,0xe8,0xe4,0xe2,0xe1,0xe0,0xdc,0xda,0xd9,0xd6,0xd5,0xd4,0xd3,0xd2,0xd1,0xce, 
                         0xcd,0xcc,0xcb,0xca,0xc9,0xc7,0xc6,0xc5,0xc3,0xbc,0xba,0xb9,0xb6,0xb5,0xb4,0xb3, 
                         0xb2,0xb1,0xae,0xad,0xac,0xab,0xaa,0xa9,0xa7,0xa6,0xa5,0xa3,0x9f,0x9e,0x9d,0x9b, 
                         0x98,0x97,0x90,0x8f,0x88,0x84,0x82,0x81,0x80,0x7c,0x7a,0x79,0x76,0x75,0x74,0x73, 
                         0x72,0x71,0x6e,0x6d,0x6c,0x6b,0x6a,0x69,0x67,0x66,0x65,0x63,0x5c,0x5a,0x59,0x56, 
                         0x55,0x54,0x53,0x52,0x51,0x4f,0x4d,0x4c,0x4b,0x4a,0x49,0x47,0x46,0x45,0x43,0x3c, 
                         0x3a,0x39,0x36,0x35,0x34,0x33,0x32,0x31,0x2e,0x2d,0x2c,0x2b,0x2a,0x29,0x27,0x26, 
                         0x25,0x23,0x1f,0x1e,0x1d,0x1b,0x18,0x17,0x10,0x0f,0x08,0x04,0x02,0x01,0x00 }; 

    /** The FibreInterface constructor.
      * @param topology;
      * @param channel
      * @param hardAddress
      * @param loopId
      * @param portId
      * @param WorldWidePortName
      * @param WorldWideNodeName
      */
    public FibreInterface (int topology,          
                           int channel,
                           int hardAddress,
                           int loopId,
                           String portId,
                           String nodeName,
                           String portName)
    {
        this.topology = topology;
        this.channel = channel;
        this.hardAddress = hardAddress;
        this.loopId = loopId;
        this.portId = portId;
        this.nodeName = nodeName;
        this.portName = portName;
    }

    /** Get the topology
      * @return the topology
      */
    public int getTopology() 
    {
        return topology;
    }

    /** Get the channel
      * @return the channel
      */
    public int getChannel() 
    {
        return channel;
    }

    /** Get the hardAddress
      * @return the hardAddress
      */
    public String getHardAddress() 
    {
        int address = getAlpa(hardAddress);

        if (address == NOT_DEFINED)
            return JCRMUtil.getNLSString("valueAddressNotApplicable");
        else 
            return hardAddress + "/" + "0x" + Integer.toHexString(address).toUpperCase();
    }

    /** Get the loopId
      * @return the loopId
      */
    public int getLoopId() 
    {
        return loopId;
    }

    /** Get the portId
      * @return the portId
      */
    public String getPortId() 
    {
        return portId;
    }

    /** Get the current Id (loopId + AL_PA)
      * @return the current Id (loopId + AL_PA)
      */
    public String getCurrentId() 
    {
        int address = getAlpa(loopId);

        if (address == NOT_DEFINED)
            return JCRMUtil.getNLSString("valueAddressNotApplicable");
        else 
            return loopId + "/" + "0x" + (Integer.toHexString(address).toUpperCase());
    }

    /** Get the worldWidePortName
      * @return the worldWidePortName
      */
    public String getPortName() 
    {
        return portName;
    }

    /** Get the worldWideNodeName
      * @return the worldWideNodeName
      */
    public String getNodeName() 
    {
        return nodeName;
    }

    public static int getAlpa(int loopID) 
    { 
       if (loopID >= 0 && loopID < loopIdTable.length) 
          return loopIdTable[loopID]; 
       else 
          return NOT_DEFINED; 
    } 
}
