package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageComponent.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                      DAK           02/07/02
//

import java.io.*;
import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;

public class StorageComponent implements Constants, Serializable 
{
    static final long serialVersionUID = -4324322171543099182L;

    // Component Types
    public static final int BATTERY = 1;
    public static final int FAN = 2;
    public static final int GBIC = 3;
    public static final int MINIHUB = 4;
    public static final int POWER_SUPPLY = 5;
    public static final int THERMAL_SENSOR = 6;
    public static final int ENVIRONMENTAL_CARD = 7;

    // Component Status values
    public static final int BATTERY_LOST_AC = 1;
    public static final int BATTERY_NEAR_EXPIRATION = 2;
    public static final int FAILED = 3;
    public static final int MAX_TEMPERATURE_EXCEEDED = 4;
    public static final int NOMINAL_TEMPERATURE_EXCEEDED = 5;
    public static final int OPTIMAL = 6;
    public static final int REMOVED = 7;

    private int         type;
    private int         status;
    private Object      reference;
    private Vector      gbics = new Vector();

    /** The StorageComponent constructor.
      * @param type
      * @param status
      * @param productInfo
      * @param reference - a reference to this object
      */
    public StorageComponent (int type,
                             int status,
                             Object reference)
    {
        this.type = type;
        this.status = status;
        this.reference = reference;
    }

    /** Get the type
      * @return the type
      */
    public int getType() 
    {
        return type;
    }

    /** Get the status
      * @return the status
      */
    public int getStatus() 
    {
        return status;
    }

    /** Get the reference to this component
      * @return the reference to this component
      */
    public Object getReference() 
    {
        return reference;
    }

    /** Add a gbic to this component
      * @param the gbic to add
      */
    public void setGbic(StorageComponent gbic)
    {
        gbics.addElement(gbic);
    }

    /** Returns an enumeration of the gbics attached to this component
      * @return an enumeration of the gbics attached to this component
      */
    public Enumeration getGbics()
    {
        return gbics.elements();
    }
    
    /** Get the display status
      * @return the display status
      */
    public String getDisplayStatus() 
    {
        switch (getStatus()) 
        {
            case BATTERY_LOST_AC:
                return JCRMUtil.getNLSString("valueEnclosureBatteryInUse");
            case BATTERY_NEAR_EXPIRATION:
                return JCRMUtil.getNLSString("valueEnclosureBatteryNearExpire");
            case MAX_TEMPERATURE_EXCEEDED:
                return JCRMUtil.getNLSString("valueEnclosureMaxTemperatureExceeded");
            case NOMINAL_TEMPERATURE_EXCEEDED:
                return JCRMUtil.getNLSString("valueEnclosureNominalTemperatureExceeded");
            case OPTIMAL:                    
                return JCRMUtil.getNLSString("valueComponentOptimal");
            case REMOVED:                    
                return JCRMUtil.getNLSString("valueComponentRemoved");
            default:
                return JCRMUtil.getNLSString("valueComponentFailed");
        }
    }

    /** Get the overall status
      * @return the overall status, NORMAL, CRITICAL, FATAL
      */
    public int getOverallStatus() 
    {
        switch (getStatus()) 
        {
            case BATTERY_LOST_AC:
                return CRITICAL;
            case BATTERY_NEAR_EXPIRATION:
                return CRITICAL;
            case MAX_TEMPERATURE_EXCEEDED:
                return FATAL;
            case NOMINAL_TEMPERATURE_EXCEEDED:
                return CRITICAL;
            case OPTIMAL:                    
                return NORMAL;
            case REMOVED:                    
                return NORMAL;
            default:
                return FATAL;
        }
    }
}

