package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocHardDrive.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/05/02

import java.rmi.*;
import java.util.*;
import java.io.*;
import com.ibm.sysmgt.storage.api.*;
import com.ibm.sysmgt.raidmgr.util.*;    
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.GUIDataProc;
import com.ibm.sysmgt.raidmgr.wizard.raidcfg.config.*;
import com.sun.wbem.cim.*;

public class IrocHardDrive extends HardDrive
{
   static final long serialVersionUID = 5034329114896721362L;

   public static final int PRIMARY = 1;
   public static final int SECONDARY = 2;
   public static final int STRIPED = 3;
   
   /** True if this hard drive contains an OS-level partition. */
   private boolean hasOSPartition;

   /** The ID the IROC firmware assigns to any drive that is part of its RAID configuration. */
   private int physicalDiskNumber = NOT_DEFINED;

   private long arrayUniqueID;

   public IrocHardDrive(Adapter adapterIN,
                       Channel channelIN,
                       int     deviceIDIN,
                       int     sizeIN,
                       int     stateIN,
                       String  vendorIN,
                       String  modelIN,
                       String  serialnumIN,
                       String  fwlevelIN,
                       boolean pfaErrorIN,
                       int     writeCacheEnableIN,
                       boolean writeCacheEnableSupportedIN,
                       boolean selfTestSupportedIN,
                       String  fruNumberIN,
                       boolean hasOSPartitionIN,
                       long    arrayUniqueIDIN)          
   {
      // Call the superclass constructor 
      super( adapterIN,
             channelIN,
             deviceIDIN,
             sizeIN,
             stateIN,
             vendorIN, 
             modelIN, 
             serialnumIN, 
             fwlevelIN, 
             pfaErrorIN,
             writeCacheEnableIN,
             writeCacheEnableSupportedIN,
             selfTestSupportedIN,
             fruNumberIN );

       hasOSPartition = hasOSPartitionIN;
       arrayUniqueID = arrayUniqueIDIN;
   }

   /** Returns whether this drive contains an OS partition.
     * @return whether this drive contains an OS partition
     */
   public boolean hasOSPartition()
   {
      return hasOSPartition;
   }

   /** Returns the ID the IROC firmware assigns to any drive that is part of its RAID configuration.
     * This is needed for actions such as 'Remove defunct drive' that only pertain to drives under
     * the control of IROC IM firmware.
     * @return the ID the IROC firmware assigns to any drive that is part of its RAID configuration
     */
   public int getPhysicalDiskNumber()
   {
      return physicalDiskNumber;
   }

   /** Sets the ID the IROC firmware assigns to any drive that is part of its RAID configuration.
     * This is needed for actions such as 'Remove defunct drive' that only pertain to drives under
     * the control of IROC IM firmware.
     * @param physicalDiskNumberIN, the ID the IROC firmware assigns to any drive that is part of its RAID configuration
     */
   protected void setPhysicalDiskNumber(int physicalDiskNumberIN)
   {
      physicalDiskNumber = physicalDiskNumberIN;
   }

   public long getArrayUniqueID()
   {
      return arrayUniqueID;
   }

   public void setArrayUniqueID(long arrayUniqueIDIN)
   {
      arrayUniqueID = arrayUniqueIDIN;
   }

   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      Vector actions = new Vector(16);

      actions.addElement(new HelpNowAction(this));
      actions.addElement(new NullAction());
      actions.addElement(new SetToHotSpareAction(this));
      actions.addElement(new SetToReadyAction(this));
      actions.addElement(new SetToEmptyAction(this));
      actions.addElement(new SetToDefunctAction(this));
      //actions.addElement(new SetToOnlineAction(this));
      actions.addElement(new HardDriveWriteCacheEnableAction(this));
      actions.addElement(new NullAction());
      actions.addElement(new IdentifyDriveAction(getAdapter(), IdentifyDriveAction.ID_PHY_DRV, getChannelID(), getDeviceID()));

      return actions;
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public Vector getDisplayDataSet()
   {
      // copy data points from the PhysicalDevice superclass
      Vector v = getPhysicalDeviceDataSet();

      v.addElement(new Object[] { new ToolTipString("infoPhysDriveSize"), String.valueOf(getSize())});
      v.addElement(new Object[] { new ToolTipString("infoPhysDriveState"), new StatusString(getDisplayState(), getOverallStatus())});

      if ((getArray() != null) && ((getState() == ONL) || (getState() == RBL) || (getState() == DDD)))
         v.addElement(new Object[] { new ToolTipString("infoArrayLetter"), getArray().getDisplayID()});

      if (isAssignedSpare())
      {
         for (Iterator i = getSparedArrays(); i.hasNext();)
         {
            Array sparedArray = (Array) i.next();
            v.addElement(new Object[] { new ToolTipString("infoPhysDriveSparedArray"), sparedArray.getShortDisplayName()} );
         }
      }

      StatusString pfaString = (hasPFAError()) ? 
                               new StatusString(JCRMUtil.getNLSString("yes"), CRITICAL) :
                               new StatusString(JCRMUtil.getNLSString("no"));
      v.addElement(new Object[] { new ToolTipString("infoPhysDrivePFA"), pfaString});

      switch (getWriteCacheEnableStatus())
      {
         case WCE_ENABLED:
            v.addElement(new Object[] { new ToolTipString("infoPhysDriveWCE"), JCRMUtil.getNLSString("infoPhysDriveWriteBack")} );
            break;
         case WCE_DISABLED:
            v.addElement(new Object[] { new ToolTipString("infoPhysDriveWCE"), JCRMUtil.getNLSString("infoPhysDriveWriteThru")} );
            break;
         case WCE_NOTAVAILABLE:
            v.addElement(new Object[] { new ToolTipString("infoPhysDriveWCE"), JCRMUtil.getNLSString("notAvailable")} );
            break;
         case WCE_NOTSUPPORTED:
         default:
            break;
      }

      return v;
   }

   /** Returns true if this object has data that should be retrieved just before being displayed.
     * @return true if this object has dynamic data
     */
   public boolean hasDeferredData()
   {
      return false;
   }

   /** Returns true if this object has data that may change without user intervention
     * and that should be updated in a GUI display.  A hard drive object contains dynamic
     * data if it supports self tests.
     * @return true if this object has dynamic data
     */
   public boolean hasDynamicData()
   {
      return false;
   }

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      return "helpIROCHardDrive";
   }

   /** Determine equivalence.
     * @param object  The object to compare to this one
     * @return  True if the objects are functionally equivalent, false otherwise.
     */
   public boolean equals(Object object)
   {
      // Do this first, since it's fast.
      if (this == object)
         return true;

      // We DON'T subclass java.lang.Object, so we must check for superclass equality.
      if (!super.equals(object))
         return false;

      // Finally, we can do the cast and compare relevant fields.
      // nothing to compare

      // Everything matched, so we're equal!
      return true;
   }

   /** Since we override equals(), we must override hashcode().
     * @return a hash value for this object
     */
   public int hashCode()
   {
      int result = 0;
      result ^= super.hashCode();
      return result;
   }

   /** Clones this object.
     * @return a clone of this object
     */
   public Object clone()
   {
      return(IrocHardDrive) super.clone();
   }
}
